@@ -5,6 +5,7 @@
 
 from typing import Any
 
+from doctr.file_utils import is_tf_available, is_torch_available
 from .core import DetectionPredictor
 from ..preprocessor import PreProcessor
 from .. import detection
@@ -12,7 +13,11 @@ from .. import detection
 
 __all__ = ["detection_predictor"]
 
-ARCHS = ['db_resnet50', 'linknet']
+
+if is_tf_available():
+    ARCHS = ['db_resnet50', 'linknet16']
+elif is_torch_available():
+    ARCHS = ['db_resnet34', 'db_resnet50', 'db_mobilenet_v3', 'linknet16']
 
 
 def _predictor(arch: str, pretrained: bool, **kwargs: Any) -> DetectionPredictor:
